pub struct RsVectorBuilder { /* private fields */ }
A builder for FastBitVector. This is used to efficiently construct a BitVector by appending bits to it. Once all bits have been appended, the BitVector can be built using the build method. If the number of bits to be appended is known in advance, it is recommended to use with_capacity to avoid re-allocations. If bits are already available in little endian u64 words, those words can be appended using append_word.

Implementations

source

impl RsVectorBuilder

source

pub fn new() -> RsVectorBuilder

Create a new empty BitVectorBuilder.

source

pub fn with_capacity(capacity: usize) -> RsVectorBuilder

Create a new empty BitVectorBuilder with the specified initial capacity to avoid re-allocations. The capacity is measured in bits

source

pub fn append_bit<T>(&mut self, bit: T)where T: Into<u64>,

Append a bit to the vector.

source

pub fn append_word(&mut self, word: u64)

Append a word to the vector. The word is assumed to be in little endian, i.e. the least significant bit is the first bit.

source

impl RsVectorBuilder

A trait to be implemented for each bit vector type to specify how it is built from a BitVectorBuilder.

source

pub fn build(self) -> RsVec

Build the BitVector from all bits that have been appended so far. This will consume the BitVectorBuilder.

source

pub fn from_bit_vec(vec: BitVec) -> RsVec

Build a BitVector from a BitVec. This will consume the BitVec.