This module contains a simple bit vector implementation with no overhead and a fast succinct bit vector implementation with rank and select queries.
Modules
- A fast succinct bit vector implementation with rank and select queries. Rank computes in constant time, select on average in constant time, with a logarithmic worst case.
Structs
- A simple bit vector that does not support rank and select queries. It stores bits densely in 64 bit limbs. The last limb may be partially filled. Other than that, there is no overhead.
- An owning iterator for
BitVec. This struct is created by theinto_itertrait implementation ofBitVec. - A borrowing iterator for
BitVec. This struct is created by theitermethod ofBitVec.