Struct vers_vecs::bit_vec::BitVec

source ·
pub struct BitVec { /* private fields */ }
A simple bit vector that does not support rank and select queries. It has a constant memory overhead of 32 bytes on the stack.

Implementations

source

impl BitVec

source

pub fn new() -> Self

Create a new empty bit vector.

source

pub fn with_capacity(capacity: usize) -> Self

Create a new empty bit vector with the given capacity. The capacity is measured in bits.

source

pub fn from_zeros(len: usize) -> Self

Create a new bit vector with all zeros and the given length. The length is measured in bits.

source

pub fn append(&mut self, bit: bool)

Append a bit to the bit vector.

source

pub fn truncate(&mut self, n: usize)

Drop the last n bits from the bit vector.

source

pub fn append_bit(&mut self, bit: u64)

Append a bit from a quad-word. The least significant bit is appended to the bit vector. All other bits are ignored.

source

pub fn append_word(&mut self, word: u64)

Append a word to the bit vector. The least significant bit is appended first.

source

pub fn append_bits(&mut self, bits: u64, len: usize)

Append multiple bits to the bit vector. The least significant bit is appended first. The number of bits to append is given by len. The bits are taken from the least significant bits of bits. All other bits are ignored.

source

pub fn len(&self) -> usize

Return the length of the bit vector. The length is measured in bits.

source

pub fn is_empty(&self) -> bool

Return whether the bit vector is empty (contains no bits).

source

pub fn flip_bit(&mut self, pos: usize)

Flip the bit at the given position.

source

pub fn get(&self, pos: usize) -> bool

Return the bit at the given position.

source

pub fn get_bits(&self, pos: usize, len: usize) -> u64

Return multiple bits at the given position. The number of bits to return is given by len. At most 64 bits can be returned.

source

pub fn heap_size(&self) -> usize

Returns the number of bytes on the heap for this vector. Does not include allocated memory that isn’t used.