Type Alias vers_vecs::bit_vec::BitMask

source ·
pub type BitMask<'s, 'b> = MaskedBitVec<'s, 'b, fn(_: u64, _: u64) -> u64>;
Type alias for masked bitvectors that implement a simple bitwise binary operation. The first lifetime is for the bit vector that is being masked, the second lifetime is for the mask.

Aliased Type

struct BitMask<'s, 'b> { /* private fields */ }